﻿use math as m,array as a

//dot product x-dot-y

function dotprod(x,y)
	if len(x)#len(y) then
		throw 'error in dotprod: vectors must be same length';
	end;
	ret=0;
	for i=0 to len(x)-1 do
		ret+=x[i][0]*y[i][0];
	end;
	return ret;
end;

//make zero matrix
function zeros(cols,rows)
	name=a.create(cols,0);
	for i=0 to len(name)-1 do
		name[i]=a.create(rows,0);
	end;
	return name;
end;

function ones(cols,rows)
	name=a.create(cols,1);
	for i=0 to len(name)-1 do
		name[i]=a.create(rows,1);
	end;
	return name;
end;

//matrix dimension [cols,rows]
function dim(a)
	return [len(a),len(a[0])];
end;


function printMatrix(a)
	print 'matrix,',dim(a),'=';
	for i=0 to len(a)-1 do
		print a[i];
	end;
	print '';
end;

//identity matrix

function eye(dim)
	ret=zeros(dim,dim);
	for i=0 to len(ret)-1 do
		ret[i][i]=1;
	end;
	return ret;
end;

//matrix multiplication a*b
//array=column vector
function matrixMul(a,b)

	if isnum(a) then
		ret=zeros(dim(b)[0],dim(b)[1]);
		for i=0 to dim(ret)[0]-1 do
			for j=0 to dim(ret)[1]-1 do
				ret[i][j]=b[i][j]*a;
			end;
		end;
		return ret;
	end;

	if isnum(b) then
		ret=zeros(dim(a)[0],dim(a)[1]);
		for i=0 to dim(ret)[0]-1 do
			for j=0 to dim(ret)[1]-1 do
				ret[i][j]=a[i][j]*b;
			end;
		end;
		return ret;
	end;


	if dim(a)[1]#dim(b)[0] then
		print 'error, tried to multiply',dim(a),'*',dim(b);
		throw 'matrixMul: matrix dimensions must agree';
	end;

	ret=zeros(dim(a)[0],dim(b)[1]);

	for i=0 to dim(ret)[0]-1 do
		for j=0 to dim(ret)[1]-1 do
			for k=0 to dim(a)[1]-1 do
				ret[i][j]+=a[i][k]*b[k][j];
			end;
		end;
	end;
	return ret;
end;

function add(a,b)
	if dim(a)[0]#dim(b)[0] and dim(a)[1]#dim(b)[1] then
		throw 'cannot add,matrix dimensions must agree';
	end;
	ret=zeros(dim(a)[0],dim(a)[1]);

	for i=0 to dim(a)[0]-1 do
		for j=0 to dim(a)[1]-1 do
			ret[i][j]=a[i][j]+b[i][j];
		end;
	end;
	return ret;
end;


//skew symmetric matrix
function skew(v)
	if dim(v)[0]#3 and dim(v)[1]#1 then
		throw 'cannot create skew symmetric matrix';
	end;
	ret=zeros(3,3);
	ret[0][1]=-v[2][0];
	ret[0][2]=v[1][0];
	ret[1][0]=v[2][0];
	ret[1][2]=-v[0][0];
	ret[2][0]=-v[1][0];
	ret[2][1]=v[0][0]; 
	return ret;
end;

//cross product a-x-b
function crossProd(a,b)
	ax=skew(a);
	return matrixMul(ax,b);
end;

function transpose(a)
	trans=zeros(dim(a)[1],dim(a)[0]);

	for i=0 to dim(a)[0]-1 do
		for j=0 to dim(a)[1]-1 do
			trans[j][i]=a[i][j];
		end;
	end;
	return trans;
end;

//classical euler representation
function genRot(ax,th)
	x=ax[0][0];
	y=ax[1][0];
	z=ax[2][0];
	rot=zeros(3,3);
	v=1-m.cos(th);
	s=m.sin(th);
	c=m.cos(th);

	rot[0][0]=x*x*v+c;
	rot[0][1]=x*y*v-z*s;
	rot[0][2]=x*z*v+y*s;
	rot[1][0]=x*y*v+z*s;
	rot[1][1]=y*y*v+c;
	rot[1][2]=y*z*v-x*s;
	rot[2][0]=x*z*v-y*s;
	rot[2][1]=y*z*v+x*s;
	rot[2][2]=z*z*v+c;
	return rot;
end;


function abs(v)
	if dim(v)[0]#1 and dim(v)[1]#1 then
		throw 'error in abs:  must be vector';
	end;

	if dim(v)[0]#1 then
		ret=0;
		for i=0 to len(v)-1 do
			ret+=v[i][0]*v[i][0];
		end;
	end;

	if dim(v)[1]#1 then
		ret=0;
		for i=0 to len(v[0])-1 do
			ret+=v[0][i]*v[0][i];
		end;
	end;
	return m.sqrt(ret);
end;


function sub(a,b)
	if dim(a)[0]#dim(b)[0] and dim(a)[1]#dim(b)[1] then
		throw 'cannot add,matrix dimensions must agree';
	end;
	ret=zeros(dim(a)[0],dim(a)[1]);

	for i=0 to dim(a)[0]-1 do
		for j=0 to dim(a)[1]-1 do
			ret[i][j]=a[i][j]-b[i][j];
		end;
	end;
	return ret;
end;

function round(a)
	ret=zeros(dim(a)[0],dim(a)[1]);

	for i=0 to dim(a)[0]-1 do
		for j=0 to dim(a)[1]-1 do
			ret[i][j]=m.round(10*a[i][j])/10;
		end;
	end;
return ret;
end;






function equals(a,b)
if dim(a)[0]#dim(b)[0] or dim(a)[1]#dim(b)[1] then
throw 'dim error in equals';
end;
for i=0 to len(a)-1 do
for j=0 to len(a[0])-1 do
if a[i][j]#b[i][j] then
 return false; 
end;
end;
end;
return true;
end;


function roundM(a)
for i=0 to len(a)-1 do
for j=0 to len(a[0])-1 do 
try
 a[i][j]=m.round(a[i][j]*100)/100;
catch e by
print 'dpp';
print a[i][j];
end;
end;
end;
return a;
end;

